# Implements the BinectApi
class Binectrest::BinectApi
    include ActiveModel::Model

    # Accessor
    attr_accessor :login,
        :password,
        :is_error,
        :error,
        :fault_code,
        :api_wsdl_url


    validates :login,       presence: true #, length: {in:2..255}
    validates :password,    presence: true

    # Constructor
    # @param [Hash] hash It must contain :login and :password
    def initialize(hash)
        super(hash.keep_if { |key,value| [:login, :password, :api_wsdl_url].include? key })
        self.is_error = false

        # TODO: Enable SSL-Verification
        @client = Savon.client(wsdl: hash[:api_wsdl_url] || "https://app.binect.de/ws/services/lettershop?wsdl", basic_auth: [ hash[:login], hash[:password] ], ssl_verify_mode: :none)
    end

    # Return the available balance
    # Question: how to convert the balance to €?
    # @return [HASH] with the given keys :whole_balance, :real_balance and :action_balance=>"500"
    def balance
        response = @client.call(:get_balance)
        self.is_error = false
        response.body[:get_balance_response][:balance]
    end

    # Upload file to server.
    # The method will immediately upload the file. It returns the order_id if the document validation succeeds. Otherwise
    # is_error will be true and the fault_code is returned
    # @param [String] filename
    # @param [Object] file_data binary file that should be uploaded
    # @return [String] order_id or fault_code
    def upload(filename, file_data)
        enc_file_data   = Base64.encode64(file_data)
        message         = { filename: filename, fileContent: enc_file_data }
        begin
            response      = @client.call(:upload, message: message)
            self.is_error = false
            response.body[:upload_response][:order_id]
        rescue Savon::SOAPFault => exception
            self.is_error = true
            self.error = exception.to_hash[:fault][:detail][:lettershop_fault]
            self.fault_code = self.error[:fault_code] if self.error[:fault_code].present?
            self.fault_code
        end
    end

    # Delete a previous stored order
    # @param order_id [String] the Id of the order to delete. @see upload
    # @return [bool] true or false if error occured
    def delete_order(order_id)
        begin
            @client.call(:delete_order, message: { orderID: order_id } )
            self.is_error = false
            true
        rescue Savon::SOAPFault => exception
            self.is_error = true
            self.error = exception.to_hash[:fault][:detail][:lettershop_fault]
            self.fault_code = self.error[:fault_code] if self.error[:fault_code].present?
            false
        end
    end

    # Get preview image (PNG).
    # The method will return an image as PNG that shows the document.
    # @param [String] the Id of the order. @see upload
    # @return [String] order_id or fault_code
    def get_preview_image(order_id)
        begin
            response = @client.call(:get_preview_png, message: { orderID: order_id } )
            base_64_encoded_image = response.body[:get_preview_png_response][:file_content]
            self.is_error = false
            Base64.decode64(base_64_encoded_image)
        rescue Savon::SOAPFault => exception
            self.is_error = true
            self.error = exception.to_hash[:fault][:detail][:lettershop_fault]
            self.fault_code = self.error[:fault_code] if self.error[:fault_code].present?
            self.fault_code
        end
    end

    # Cancel an order in process.
    # @param order_id [String] the Id of the order to cancel. @see upload
    # @return [bool] true or false if error occured
    def cancel_order(order_id)
        begin
            @client.call(:cancel_order, message: { orderID: order_id } )
            self.is_error = false
            true
        rescue Savon::SOAPFault => exception
            self.is_error = true
            self.error = exception.to_hash[:fault][:detail][:lettershop_fault]
            self.fault_code = self.error[:fault_code] if self.error[:fault_code].present?
            false
        end
    end

    # Get information of an order.
    # @param oder_id [String] the Id of the order to cancel. @see upload
    # @return [HASH] Data about validation state, price, ..
    def get_order(order_id)
        begin
            response = @client.call(:get_order, message: { orderID: order_id } )
            self.is_error = false
            response.body[:get_order_response][:order]
        rescue Savon::SOAPFault => exception
            self.is_error = true
            self.error = exception.to_hash[:fault][:detail][:lettershop_fault]
            self.fault_code = self.error[:fault_code] if self.error[:fault_code].present?
            false
        end
    end

    # Returns the price of a order.
    # @param oder_id [String] the Id of the order to cancel. @see upload
    # @return [Hash] with prices before and after tax
    def get_order_price(order_id)
        begin
            response = @client.call(:get_order_price, message: { orderID: order_id } )
            self.is_error = false
            response.body[:get_order_price_response][:amount]
        rescue Savon::SOAPFault => exception
            self.is_error = true
            self.error = exception.to_hash[:fault][:detail][:lettershop_fault]
            self.fault_code = self.error[:fault_code] if self.error[:fault_code].present?
            false
        end
    end

    # Returns a price for given meta-data.
    # @param number_of_pages [Integer]
    # @param is_international [Bool] it is an international letter.
    # @param is_simplex [Bool] simplex printing or duplex
    # @param is_color [Bool] print to color
    # @param envelope [String] the envelope (like "C4", default: "C5") (optional)
    # @param recipient_address [String] ?? (optional)
    # @return [Float] Price in Cent.
    def get_price(number_of_pages, is_international, is_simplex, is_color, envelope = nil, recipient_address = nil)
        response = @client.call(:get_price, message: { OrderMetaData: { recipientAdress: recipient_address, numberOfPages: number_of_pages, isInternational: is_international, :options => { simplex: is_simplex, color: is_color, envelope: envelope } } })
        self.is_error = false
        response.body[:get_price_response][:amount]
    rescue Savon::SOAPFault => exception
        self.is_error = true
        self.error = exception.to_hash[:fault][:detail][:lettershop_fault]
        self.fault_code = self.error[:fault_code] if self.error[:fault_code].present?
        false
    end

    # Send a letter.
    # @param order_id [String] the Id of the order to send. @see upload
    # @return [bool] true or false if error occured
    def send_order(order_id)
        begin
            @client.call(:send_order, message: { orderID: order_id } )
            self.is_error = false
            true
        rescue Savon::SOAPFault => exception
            self.is_error = true
            self.error = exception.to_hash[:fault][:detail][:lettershop_fault]
            self.fault_code = self.error[:fault_code] if self.error[:fault_code].present?
            false
        end
    end

    # Send a whole shopping cart
    # @param order_id [String] the Id of the order to cancel. @see upload
    # @return [bool] true or false if error occured
    def send_whole_shopping_cart
        begin
            @client.call(:send_whole_shopping_cart)
            self.is_error = false
            true
        rescue Savon::SOAPFault => exception
            self.is_error = true
            self.error = exception.to_hash[:fault][:detail][:lettershop_fault]
            self.fault_code = self.error[:fault_code] if self.error[:fault_code].present?
            false
        end
    end

    # Validate the letter.
    # @param order_id [String] the Id of the order to cancel. @see upload
    # @return [bool] true or false if error occured
    def validate(order_id)
        begin
            fail "not finished" if true
            @client.call(:validate, message: { orderID: order_id })
            self.is_error = false
            true
        rescue Savon::SOAPFault => exception
            self.is_error = true
            self.error = exception.to_hash[:fault][:detail][:lettershop_fault]
            self.fault_code = self.error[:fault_code] if self.error[:fault_code].present?
            false
        end
    end

    # Apply transformation on letter.
    # @param order_id [String] the Id of the order to cancel. @see upload
    # @params [float] offset_x delta position change to fit document
    # @params [float] offset_y delta position change to fit document
    # @params [float] scale_x delta scale in %.
    # @params [float] scale_y delta scale in %.
    # @return [bool] true or false if error occured
    def transform(order_id, offset_x, offset_y, scale_x, scale_y)
        begin
            @client.call(:transform, message: { orderID: order_id, Transformation: { offsetX: offset_x, offsetY: offset_y, scaleX: scale_x, scaleY: scale_y } })
            self.is_error = false
            # There is no response defined.
            true
        rescue Savon::SOAPFault => exception
            self.is_error = true
            self.error = exception.to_hash[:fault][:detail][:lettershop_fault]
            self.fault_code = self.error[:fault_code] if self.error[:fault_code].present?
            false
        end
    end

    # Set shipping options
    def set_shipping_options(order_id, is_simplex, is_color, is_international = nil, envelope = nil)
        begin
            @client.call(:set_shipping_options, message: { orderID: order_id, Options: { simplex: is_simplex, color: is_color, envelope: envelope, isInternational: is_international } } )
            self.is_error = false
            # There is no response defined.
            true
        rescue Savon::SOAPFault => exception
            self.is_error = true
            self.error = exception.to_hash[:fault][:detail][:lettershop_fault]
            self.fault_code = self.error[:fault_code] if self.error[:fault_code].present?
            false
        end
    end

    def register_user(e_mail, password)
        begin
            @client.call(:register_user, message: { User: { email: e_mail }, password: password } )
            self.is_error = false
            # There is no response defined.
            true
        rescue Savon::SOAPFault => exception
            self.is_error = true
            self.error = exception.to_hash[:fault][:detail][:user_fault]
            self.fault_code = self.error[:fault_code] if self.error[:fault_code].present?
            false
        end
    end
end
